/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
#include "letterwidget.h"
#include <QRect>
#include <QPainter>
#include <QPainterPath>
#include <QPolygon>

letterwidget::letterwidget(QGraphicsItem *parent)
    : HbWidget(parent)
{
    mItalic = new HbPushButton("Italic");
    mBold = new HbPushButton("Bold");
    mUnderline = new HbPushButton("Underline");
    mRed = new HbPushButton("Red");
    mGreen = new HbPushButton("Green");
    mBlue = new HbPushButton("Blue");
    mSave = new HbPushButton("Save activity");
    mDelete = new HbPushButton("Delete activity");
    
    mVisibilityLabel = new HbLabel("Visibility");
    mVisibilityCombo = new HbComboBox();
    mVisibilityCombo->addItem("none");
    mVisibilityCombo->addItem("true");
    mVisibilityCombo->addItem("false");
    
    mPersistentLabel = new HbLabel("Persistence");
    mPersistentCombo = new HbComboBox();
    mPersistentCombo->addItem("none");
    mPersistentCombo->addItem("true");
    mPersistentCombo->addItem("false");
    
    mNameLabel = new HbLabel("Name");
    mNameCombo = new HbComboBox();
    mNameCombo->addItem("none");    
    
    mGridLayout = new QGraphicsGridLayout();
    mGridLayout->addItem(mRed, 3, 0, 1, 1);
    mGridLayout->addItem(mGreen, 3, 1, 1, 1);
    mGridLayout->addItem(mBlue, 3, 2, 1, 1);
    mGridLayout->addItem(mSave, 4, 0, 1, 1);
    mGridLayout->addItem(mDelete, 4, 1, 1, 1);
    mGridLayout->addItem(mVisibilityLabel, 5, 0, 1, 1);
    mGridLayout->addItem(mVisibilityCombo, 5, 1, 1, 2);
    mGridLayout->addItem(mPersistentLabel, 6, 0, 1, 1);
    mGridLayout->addItem(mPersistentCombo, 6, 1, 1, 2);
    mGridLayout->addItem(mNameLabel, 7, 0, 1, 1);
    mGridLayout->addItem(mNameCombo, 7, 1, 1, 2);
    mGridLayout->setRowStretchFactor(0, 1);
    mGridLayout->setRowStretchFactor(1, 1);
    setLayout(mGridLayout);


    connect(mRed, SIGNAL(released()), this, SLOT(red()));
    connect(mGreen, SIGNAL(released()), this, SLOT(green()));
    connect(mBlue, SIGNAL(released()), this, SLOT(blue()));

    connect(mItalic, SIGNAL(released()), this, SIGNAL(toitalic()));
    connect(mBold, SIGNAL(released()), this, SIGNAL(tobold()));
    connect(mUnderline, SIGNAL(released()), this, SIGNAL(tounderline()));

    connect(mSave, SIGNAL(released()), this, SIGNAL(save()));
    connect(mDelete, SIGNAL(released()), this, SIGNAL(del()));
}

void letterwidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    HbWidget::paint(painter, option, widget);

    if (mColor == "red") {
        painter->setPen(Qt::red);
    } else if (mColor == "green") {
        painter->setPen(Qt::green);
    } else if (mColor == "blue") {
        painter->setPen(Qt::blue);
    } else {
        Q_ASSERT(false);
    }
}

letterwidget::~letterwidget()
{
}


void letterwidget::red()
{
    mColor = "red";
    update();
}

void letterwidget::green()
{
    mColor = "green";
    update();
}

void letterwidget::blue()
{
    mColor = "blue";
    update();
}

void letterwidget::setcolor(const QString &color)
{
    mColor = color;
}

QString letterwidget::color()
{
    return mColor;
}

QString letterwidget::visibility()
{
    return mVisibilityCombo->currentText();
}

QString letterwidget::persistent()
{
	return mPersistentCombo->currentText();
}

QString letterwidget::name()
{
    return mNameCombo->currentText();
}
