/*
* Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:
*
*/
/*
 * T_casathandler.cpp
 *
 *  Created on: 2009-09-16
 *      Author:
 */


#include "cadef.h"
#include "ca2internalCRkeys.h"
#include "T_casathandler.h"
#include "casathandler.h"
#include "e32property.h"


// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
//
void T_CaSatHandler::testConstruction()
{
    bool constructed(false);
    TRAPD(err,
          CCaSatHandler *satHandler  = CCaSatHandler::NewL();
          CleanupStack::PushL(satHandler);

          constructed = satHandler != NULL;

          CleanupStack::PopAndDestroy(satHandler);
         );
    QCOMPARE(err, KErrNone);
    QVERIFY(constructed);
}

// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
//
void T_CaSatHandler::testGetName()
{
    bool nameMatches(false);
    TRAPD(err,
          CCaSatHandler *satHandler  = CCaSatHandler::NewL();
          CleanupStack::PushL(satHandler);
          RBuf satName;
          satName.CleanupClosePushL();
          satName.Create(KCenRepBufferSize);
          RProperty::Get(KCRUidCa, KCaSatUIName, satName);
          RBuf satHandlerName;
          satHandlerName.CleanupClosePushL();
          satHandlerName.Create(KCenRepBufferSize);
          satHandler->GetName(satHandlerName);

          nameMatches  = satHandlerName.Compare(satName) == 0;

          CleanupStack::PopAndDestroy(&satHandlerName);
          CleanupStack::PopAndDestroy(&satName);
          CleanupStack::PopAndDestroy(satHandler);
         );
    QCOMPARE(err, KErrNone);
    QVERIFY(nameMatches);
}

// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
//
void T_CaSatHandler::testCheckVisibility()
{
    ;
    TInt visibility(KErrNone);
    RProperty::Get(KCRUidCa, KCaShowSatUI, visibility);
    TBool visibilityHandler = CCaSatHandler::CheckVisibility();
    QCOMPARE(visibilityHandler, visibility);
}


// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
//
void T_CaSatHandler::testGetIcon()
{
    // preconditions
    RProperty::Define( KCRUidCa, KCaSatUIIconId, RProperty::EInt );
    
    bool loadIconResult(false);
    TInt setSizeResult(KErrNone);
    TRAPD(err,
        CCaSatHandler *satHandler  = CCaSatHandler::NewL();
        CleanupStack::PushL(satHandler);
        TInt iconId(KErrNone);
        User::LeaveIfError(RProperty::Get(KCRUidCa, KCaSatUIIconId, iconId));
        if (iconId > 0) {
            CAknIcon *icon = NULL;
            icon = satHandler->LoadIconL();
            CleanupStack::PushL(icon);
            loadIconResult = (icon != NULL);
            TSize size(1, 1);
            //TInt err = AknIconUtils::SetSize( icon->Bitmap(), size ) ;
            setSizeResult = AknIconUtils::SetSize(icon->Bitmap(), size);
            CleanupStack::PopAndDestroy(icon);
        } else {
            CAknIcon *icon = NULL;
            icon = satHandler->LoadIconL();
            loadIconResult = (icon == NULL);
        }
        CleanupStack::PopAndDestroy(satHandler);
        );
    
    QCOMPARE(err, KErrNone);
    QVERIFY(loadIconResult);
    QCOMPARE(setSizeResult, KErrNone);
}

// ---------------------------------------------------------------------------
//
// ---------------------------------------------------------------------------
//
void T_CaSatHandler::testGetBitmap()
{
    CCaSatHandler *satHandler = CCaSatHandler::NewL();
    CleanupStack::PushL(satHandler);
    RIconEf iIconEf;
    CleanupClosePushL(iIconEf);
    //CFbsBitmap* bitmap = satHandler->GetBitmapL( iIconEf );
    //EUNIT_ASSERT( !bitmap );
    CleanupStack::PopAndDestroy(&iIconEf);
    CleanupStack::PopAndDestroy(satHandler);
}

QTEST_MAIN(T_CaSatHandler);
